@@ -15,24 +15,97 @@
 # define JSON_DIR "tests/json"
 #endif
 
+static ssize_t
+test_bson_json_read_cb_helper (void *string, uint8_t *buf, size_t len)
+{
+   size_t str_size = strlen ((char *) string);
+   if (str_size) {
+      memcpy (buf, string, BSON_MIN (str_size, len));
+      return str_size;
+   } else {
+      return 0; 
+   }
+}
+
+static void
+test_bson_json_allow_multiple (void)
+{
+   int32_t one;
+   bson_error_t error;
+   bson_json_reader_t *reader;
+   bson_json_reader_t *reader_multiple;
+   bson_t bson = BSON_INITIALIZER;
+   bool allow_multiple = true;
+   char *multiple_json = "{\"a\": 1}{\"b\": 1}";
+
+   reader = bson_json_reader_new (multiple_json,
+                                  &test_bson_json_read_cb_helper,
+                                  NULL, !allow_multiple, 0);
+   reader_multiple = bson_json_reader_new (multiple_json,
+                                           &test_bson_json_read_cb_helper,
+                                           NULL, allow_multiple, 0);
+   assert (reader);
+   assert (reader_multiple);
+
+   /* reader with allow_multiple false */
+   /* read first json */
+   ASSERT_CMPINT (1, ==, bson_json_reader_read (reader, &bson, &error));
+   BCON_EXTRACT (&bson, "a", BCONE_INT32 (one));
+   ASSERT_CMPINT (1, ==, one);
+
+   /* read second json */
+   bson_reinit (&bson);
+   ASSERT_CMPINT (1, ==, bson_json_reader_read (reader, &bson, &error));
+   BCON_EXTRACT (&bson, "b", BCONE_INT32 (one));
+   ASSERT_CMPINT (1, ==, one);
+
+   /* start at the beginning of the string */
+   bson_reinit (&bson);
+   ASSERT_CMPINT (1, ==, bson_json_reader_read (reader, &bson, &error));
+   BCON_EXTRACT (&bson, "a", BCONE_INT32 (one));
+   ASSERT_CMPINT (1, ==, one);
+
+   /* reader_multiple with allow_multiple true */
+   /* read first json */
+   bson_reinit (&bson);
+   ASSERT_CMPINT (1, ==, bson_json_reader_read (reader_multiple,
+                                                &bson, &error));
+   BCON_EXTRACT (&bson, "a", BCONE_INT32 (one));
+   ASSERT_CMPINT (1, ==, one);
+
+   /* read second json */
+   bson_reinit (&bson);
+   ASSERT_CMPINT (1, ==, bson_json_reader_read (reader_multiple,
+                                                &bson, &error));
+   BCON_EXTRACT (&bson, "b", BCONE_INT32 (one));
+   ASSERT_CMPINT (1, ==, one);
+
+   /* start at the beginning of the string */
+   bson_reinit (&bson);
+   ASSERT_CMPINT (1, ==, bson_json_reader_read (reader_multiple,
+                                                &bson, &error));
+   BCON_EXTRACT (&bson, "a", BCONE_INT32 (one));
+   ASSERT_CMPINT (1, ==, one);
+
+   bson_json_reader_destroy (reader);
+   bson_json_reader_destroy (reader_multiple);
+   bson_destroy (&bson);
+}
+
 
 static void
 test_bson_as_json (void)
 {
    bson_oid_t oid;
-#ifdef BSON_EXPERIMENTAL_FEATURES
    bson_decimal128_t decimal128;
-#endif
    bson_t *b;
    bson_t *b2;
    char *str;
    size_t len;
    int i;
 
-#ifdef BSON_EXPERIMENTAL_FEATURES
    decimal128.high = 0x3040000000000000ULL;
    decimal128.low  = 0x000000000000000B;
-#endif
    bson_oid_init_from_string(&oid, "123412341234abcdabcdabcd");
 
    b = bson_new();
@@ -52,9 +125,7 @@ test_bson_as_json (void)
    assert(bson_append_minkey(b, "minkey", -1));
    assert(bson_append_maxkey(b, "maxkey", -1));
    assert(bson_append_symbol(b, "symbol", -1, "var a = {};", -1));
-#ifdef BSON_EXPERIMENTAL_FEATURES
    assert(bson_append_decimal128(b, "decimal128", -1, &decimal128));
-#endif
 
    b2 = bson_new();
    assert(bson_append_int32(b2, "0", -1, 60));
@@ -154,7 +225,6 @@ test_bson_as_json_double (void)
 }
 
 
-#ifdef BSON_EXPERIMENTAL_FEATURES
 static void
 test_bson_as_json_decimal128 (void)
 {
@@ -176,7 +246,45 @@ test_bson_as_json_decimal128 (void)
    bson_free(str);
    bson_destroy(b);
 }
-#endif
+
+
+static void
+test_bson_as_json_code (void)
+{
+   bson_t code = BSON_INITIALIZER;
+   bson_t scope = BSON_INITIALIZER;
+   char *str;
+
+   assert (bson_append_code (&code, "c", -1, "function () {}"));
+   str = bson_as_json (&code, NULL);
+   ASSERT_CMPSTR (
+      str,
+      "{ \"c\" : { \"$code\" : \"function () {}\" } }");
+
+   bson_free (str);
+   bson_reinit (&code);
+
+   /* empty scope */
+   assert (BSON_APPEND_CODE_WITH_SCOPE (&code, "c", "function () {}", &scope));
+   str = bson_as_json (&code, NULL);
+   ASSERT_CMPSTR (
+      str,
+      "{ \"c\" : { \"$code\" : \"function () {}\", \"$scope\" : { } } }");
+
+   bson_free (str);
+   bson_reinit (&code);
+
+   BSON_APPEND_INT32 (&scope, "x", 1);
+   assert (BSON_APPEND_CODE_WITH_SCOPE (&code, "c", "function () {}", &scope));
+   str = bson_as_json (&code, NULL);
+   ASSERT_CMPSTR (
+      str,
+      "{ \"c\" : { \"$code\" : \"function () {}\", \"$scope\" : { \"x\" : 1 } } }");
+
+   bson_free (str);
+   bson_destroy (&code);
+   bson_destroy (&scope);
+}
 
 
 static void
@@ -433,8 +541,20 @@ test_bson_json_read(void)
    _test_bson_json_read_compare(json, 5, first, second, third, NULL);
 }
 
+static void
+test_bson_json_read_corrupt_utf8 (void)
+{
+   const char *json = "{ \"a\" : \"\x80\"}";
+   bson_error_t error = { 0 };
+
+   assert (!bson_new_from_json ((uint8_t *) json, -1, &error));
+
+   ASSERT_ERROR_CONTAINS (error, BSON_ERROR_JSON,
+                          BSON_JSON_ERROR_READ_CORRUPT_JS,
+                          "invalid bytes in UTF8 string");
+}
+
 
-#ifdef BSON_EXPERIMENTAL_FEATURES
 static void
 test_bson_json_read_decimal128(void)
 {
@@ -447,7 +567,6 @@ test_bson_json_read_decimal128(void)
 
    _test_bson_json_read_compare(json, 5, doc, NULL);
 }
-#endif
 
 static void
 test_json_reader_new_from_file (void)
@@ -531,7 +650,6 @@ test_bson_json_read_invalid_binary(void)
        " \"bin\" : { \"$binary\" : \"invalid\", \"$type\" : \"80\" } }";
    bson_t b;
    bool r;
-   char *str;
 
    r = bson_init_from_json (&b, json, -1, &error);
    assert (!r);
@@ -774,7 +892,6 @@ test_bson_json_dbref (void)
    }
 }
 
-#ifdef BSON_EXPERIMENTAL_FEATURES
 static void
 test_bson_json_number_decimal (void) {
    bson_error_t error;
@@ -795,7 +912,6 @@ test_bson_json_number_decimal (void) {
    assert (decimal128.high == 0x3040000000000000ULL);
    bson_destroy (&b);
 }
-#endif
 
 static void
 test_bson_json_inc (void)
@@ -957,6 +1073,21 @@ test_bson_as_json_spacing (void)
    bson_destroy (&d);
 }
 
+static void
+test_bson_integer_width (void)
+{
+	const char * sd = "{\"v\":-1234567890123, \"x\":12345678901234}";
+	char *match;
+	bson_error_t err;
+	bson_t *bs = bson_new_from_json((const uint8_t*)sd, strlen(sd), &err);
+
+	match = bson_as_json (bs, 0);
+	ASSERT_CMPSTR (match, "{ \"v\" : -1234567890123, \"x\" : 12345678901234 }");
+
+	bson_free (match);
+	bson_destroy(bs);
+}
+
 void
 test_json_install (TestSuite *suite)
 {
@@ -965,6 +1096,7 @@ test_json_install (TestSuite *suite)
    TestSuite_Add (suite, "/bson/as_json/int32", test_bson_as_json_int32);
    TestSuite_Add (suite, "/bson/as_json/int64", test_bson_as_json_int64);
    TestSuite_Add (suite, "/bson/as_json/double", test_bson_as_json_double);
+   TestSuite_Add (suite, "/bson/as_json/code", test_bson_as_json_code);
    TestSuite_Add (suite, "/bson/as_json/utf8", test_bson_as_json_utf8);
    TestSuite_Add (suite, "/bson/as_json/stack_overflow", test_bson_as_json_stack_overflow);
    TestSuite_Add (suite, "/bson/as_json/corrupt", test_bson_corrupt);
@@ -974,6 +1106,7 @@ test_json_install (TestSuite *suite)
    TestSuite_Add (suite, "/bson/as_json_spacing", test_bson_as_json_spacing);
    TestSuite_Add (suite, "/bson/as_json_special_keys_at_top", test_bson_json_special_keys_at_top);
    TestSuite_Add (suite, "/bson/array_as_json", test_bson_array_as_json);
+   TestSuite_Add (suite, "/bson/json/allow_multiple", test_bson_json_allow_multiple);
    TestSuite_Add (suite, "/bson/json/read", test_bson_json_read);
    TestSuite_Add (suite, "/bson/json/inc", test_bson_json_inc);
    TestSuite_Add (suite, "/bson/json/array", test_bson_json_array);
@@ -983,17 +1116,15 @@ test_json_install (TestSuite *suite)
    TestSuite_Add (suite, "/bson/json/read/invalid_json", test_bson_json_read_invalid_json);
    TestSuite_Add (suite, "/bson/json/read/bad_cb", test_bson_json_read_bad_cb);
    TestSuite_Add (suite, "/bson/json/read/invalid", test_bson_json_read_invalid);
-#ifdef BSON_EXPERIMENTAL_FEATURES
+   TestSuite_Add (suite, "/bson/json/read/corrupt_utf8", test_bson_json_read_corrupt_utf8);
    TestSuite_Add (suite, "/bson/json/read/decimal128", test_bson_json_read_decimal128);
-#endif
    TestSuite_Add (suite, "/bson/json/read/file", test_json_reader_new_from_file);
    TestSuite_Add (suite, "/bson/json/read/bad_path", test_json_reader_new_from_bad_path);
    TestSuite_Add (suite, "/bson/json/read/invalid", test_bson_json_read_invalid);
    TestSuite_Add (suite, "/bson/json/read/$numberLong", test_bson_json_number_long);
    TestSuite_Add (suite, "/bson/json/read/$numberLong/zero", test_bson_json_number_long_zero);
    TestSuite_Add (suite, "/bson/json/read/dbref", test_bson_json_dbref);
-#ifdef BSON_EXPERIMENTAL_FEATURES
    TestSuite_Add (suite, "/bson/as_json/decimal128", test_bson_as_json_decimal128);
    TestSuite_Add (suite, "/bson/json/read/$numberDecimal", test_bson_json_number_decimal);
-#endif
+   TestSuite_Add (suite, "/bson/integer/width", test_bson_integer_width);
 }
